@@ -26,5 +26,10 @@ socket_listen(char *port, char *localaddr);
 int
 socket_accept(int sock);
 
+int
+socket_connect(machine *m);
+void
+socket_continue_connect(machine *m);
+
 #endif /* __SOCKET_H__ */
 
