@@ -34,9 +34,16 @@
 #define SE "\xf0"
 #define ECHO "\x01"
 #define SUPPRESS_GO_AHEAD "\x03"
+#define STATUS "\x05"
+#define TERMINAL_TYPE "\x18"
+#define TERMINAL_SPEED "\x20"
+#define NEGOTIATE_WINDOW_SIZE "\x1f"
+#define REMOTE_FLOW_CONTROL "\x21"
+#define X_DISPLAY_LOCATION "\x23"
 #define LINEMODE "\x22"
 #define NEWENVIRON "\x27"
 #define MODE "\x01"
+#define BREAK '\xf3'
 
 /* Make a request.  Not intended to be RFC-compatible, just enough
  * to convince telnet clients to do what we want...  To do this
@@ -50,18 +57,37 @@
  * too.
  */
 void
-telnet_negotiate(int socket) {
-    char request[]=
+telnet_negotiate_server(int socket) {
+    static char request[]=
       IAC DONT ECHO
       IAC WILL ECHO
       IAC WILL SUPPRESS_GO_AHEAD
       IAC DO SUPPRESS_GO_AHEAD
+      IAC DONT TERMINAL_TYPE
+      IAC DONT TERMINAL_SPEED
+      IAC DONT X_DISPLAY_LOCATION
       IAC DONT NEWENVIRON
-      IAC WONT NEWENVIRON
+#if 0
       IAC DO LINEMODE
       IAC SB LINEMODE MODE "0" IAC SE
+#endif
       ;
-    write(socket, request, sizeof(request));
+    write(socket, request, sizeof(request)-1);
+}
+
+void
+telnet_negotiate_client(int socket) {
+    static char request[]=
+      IAC WONT ECHO
+      IAC DO SUPPRESS_GO_AHEAD
+      IAC WONT NEWENVIRON
+      IAC WONT NEGOTIATE_WINDOW_SIZE
+      IAC WONT TERMINAL_TYPE
+      IAC WONT TERMINAL_SPEED
+      IAC WONT X_DISPLAY_LOCATION
+      IAC WILL REMOTE_FLOW_CONTROL
+      ;
+    write(socket, request, sizeof(request)-1);
 }
 
 int
@@ -160,6 +186,18 @@ telnet_process_input(net_conn *nc, char *data, int len) {
     return len;
 }
 
+/* Test input data for a BREAK character */
+/* should be return strnchr(data, BREAK, len); but strnstr not std */
+int
+telnet_data_has_break(net_conn *nc, char *data, int len) {
+    char *s;
+
+    for (s=data; s < data+len; s++) {
+	if (*s == BREAK) return 1;
+    }
+    return 0;
+}
+
 /* The telnet protocol requires CR/NL instead of just NL
  * We normally deal with Unix, which just uses NL, so we need to translate.
  *
