@@ -1,7 +1,7 @@
 /* ttywatch.c -- log output from TTYs, once loosely based on robin.c from
  * Linux Application Development, by Michael K. Johnson and Erik W. Troan
  *
- * Copyright  2000, 2001 Michael K. Johnson <johnsonm@redhat.com>
+ * Copyright  2000, 2001, 2002 Michael K. Johnson <johnsonm@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -82,23 +82,23 @@ static GHashTable *accept_fds;
 static GHashTable *net_fds;
 static GHashTable *waiting_fds;
 
-static int
-machine_name_is_net(char *name) {
+static socket_type
+machine_type(char *name) {
 	char *colon, *c;
 
 	if (strchr(name, '/'))
-	    return 0;
+	    return ST_NOTASOCKET;
 	colon = strchr(name, ':');
 	if (!colon)
-	    return 0;
+	    return ST_NOTASOCKET;
 
 	/* are there only numerics following the colon? */
 	for (c = ++colon; *c; c++) {
 	    if (*c < '0' || *c > '9')
-		return 0;
+		return ST_NOTASOCKET;
 	}
 
-	return 1;
+	return ST_TELNET;
 }
 
 /* get machine* for machine name, creating a new machine* if necessary */
@@ -121,13 +121,15 @@ get_machine(char *mach) {
 	m->speed = B115200;
 	m->logspew = 80*50; /* up to at least two screen pages */
 
+	m->st = machine_type(mach);
+
 	/* defaults to devpath = /dev/<name> for symlinks unless netname */
-	if (machine_name_is_net(mach)) {
-	    m->devpath = strdup(mach);
-	} else {
+	if (m->st == ST_NOTASOCKET) {
 	    m->devpath = malloc(strlen(mach)+6);
 	    strcpy(m->devpath, "/dev/");
 	    strcat(m->devpath, mach);
+	} else {
+	    m->devpath = strdup(mach);
 	}
 
 	g_hash_table_insert(machine_names, m->name, m);
@@ -202,7 +204,7 @@ setup_one_device(gpointer data, gpointer ignore) {
 
 	open_one_logfile(m);
 
-	if (machine_name_is_net(m->devpath)) {
+	if (m->st == ST_TELNET || m->st == ST_RAWTCP) {
 	    /* connection may not be available, but we keep trying */
 	    m->dev_nc.fd = socket_connect(m);
 	    if (m->dev_nc.fd > -1 && m->ss == SS_INPROGRESS) {
@@ -505,35 +507,36 @@ main(int argc, char **argv) {
 
 	do {
 	    int i, n, r;
+	    machine *m;
+	    net_conn *nc;
+	    time_t now;
 
 	    r = poll(ufds, nfds, retry ? 1000 : -1);
 	    if ((r < 0) && (errno != EINTR)) die("poll failed unexpectedly\n");
-	    for (n = 0; n < nfds; n++) {
-#		define TTYWATCH_BUFSIZE 1024
-		char buf[TTYWATCH_BUFSIZE];
-		machine *m;
-		net_conn *nc;
-		time_t now;
 
-		if (retry) {
-		    now = time(NULL);
-		    while (retry) {
-			m = retry->data;
-			if (m->timeout_expires < now) {
-			    m->dev_nc.fd = -1;
-			    socket_connect(m);
-			    if (m->dev_nc.fd > -1) {
-				if (m->ss == SS_INPROGRESS) {
-				    g_hash_table_insert(waiting_fds, &m->dev_nc.fd, m);
-				}
-				retry = g_slist_remove(retry, m);
-			    } else {
-				/* try again significantly later */
-				m->timeout_expires = time(NULL) + 10;
+	    if (retry) {
+		now = time(NULL);
+		while (retry) {
+		    m = retry->data;
+		    if (m->timeout_expires < now) {
+			m->dev_nc.fd = -1;
+			socket_connect(m);
+			if (m->dev_nc.fd > -1) {
+			    if (m->ss == SS_INPROGRESS) {
+				g_hash_table_insert(waiting_fds, &m->dev_nc.fd, m);
 			    }
+			    retry = g_slist_remove(retry, m);
+			} else {
+			    /* try again significantly later */
+			    m->timeout_expires = time(NULL) + 10;
 			}
 		    }
 		}
+	    }
+
+	    for (n = 0; n < nfds; n++) {
+#		define TTYWATCH_BUFSIZE 1024
+		char buf[TTYWATCH_BUFSIZE];
 
 		if (!ufds[n].revents) continue;
 
@@ -635,11 +638,11 @@ main(int argc, char **argv) {
 			    remove_client (nc, &ufds, &nfds);
 			    break;
 			}
-			i = telnet_process_input(nc, buf, i);
-
 			if (telnet_data_has_break(nc, buf, i)) {
 			    send_break (nc);
 			}
+
+			i = telnet_process_input(nc, buf, i);
 			if (m->ss == SS_NOTASOCKET) {
 			    write(m->dev_nc.fd, buf, i);
 			} else {
